import { Injectable } from '@angular/core';
import { Observable, Subject, from } from 'rxjs';
import { map, concatMap } from 'rxjs/operators';
import { HttpClient } from '@angular/common/http';
import { Resource, IResource, ResourceRO, IResourceFormat } from '../models/resource.model';
import { APP_CONFIG } from './app-config.service';

@Injectable()
export class ResourceService {

  resourceServiceUrl: string;
  limit: number;
  pageNumber: number;
  sortOptions: {
    value: string,
    order: string,
  };

  private _searchChangeSubject: Subject<any>;

  constructor(
    private _httpClient: HttpClient) {
    this.resourceServiceUrl = `${APP_CONFIG.resources.url}resources/`;
    this._searchChangeSubject = new Subject<any>();
    this.limit = 10;
    this.pageNumber = 1;
  }

  getResources(): Observable<ResourceRO> {
    let query = '?';
    query += `limit=${(this.limit ? this.limit : 20)}`;
    query += `&offset=${(this.pageNumber ? (this.pageNumber - 1) * this.limit : 0)}`;
    query += `&sort_by=${this.sortOptions.value}.${this.sortOptions.order}`;

    return this._httpClient.get<IResource[]>(this.resourceServiceUrl + query, { observe: 'response' }).pipe(
      map((response) => {
        const totalCount = response.headers.get('Content-Range');
        const resources = [];
        response.body.forEach((resource) => {
          resources.push(new Resource(resource));
        });
        return new ResourceRO(resources, parseInt(totalCount, 10));
      }));
  }

  findById(id): Observable<Resource> {
    return this._httpClient.get<IResource>(this.resourceServiceUrl + id).pipe(
      map((response) => {
        return new Resource(response);
      }),
    );
  }

  delete(id) {
    return this._httpClient.delete(this.resourceServiceUrl + id, { withCredentials: true });
  }

  replaceOrCreate(data): Observable<Resource> {
    if (data.id) {
      return this._httpClient.put<IResource>(this.resourceServiceUrl + data.id, data, { withCredentials: true }).pipe(
        map((response) => {
          return new Resource(response);
        }),
      );
    }
    return this._httpClient.post<IResource>(this.resourceServiceUrl, data, { withCredentials: true }).pipe(
      map((response) => {
        return new Resource(response);
      }),
    );
  }

  createResourceFormats(resourceId: string, resourceFormats: IResourceFormat[]): Observable<IResourceFormat> {
    return from(resourceFormats).pipe(
      concatMap((rf) => {
        return this._httpClient.post<IResourceFormat>(`${this.resourceServiceUrl}${resourceId}/formats`, rf, { withCredentials: true });
      }),
    );
  }

  updateResourceFormats(resourceId: string, resourceFormats: IResourceFormat[]) {
    return from(resourceFormats).pipe(
      concatMap((rf) => {
        return this._httpClient.put<IResourceFormat>(
          `${this.resourceServiceUrl}${resourceId}/formats/${rf.id}`,
          rf,
          { withCredentials: true },
        );
      }),
    );
  }

  deleteResourceFormats(resourceId: string, resourceFormatsId: string[]) {
    return from(resourceFormatsId).pipe(
      concatMap((rfId) => {
        return this._httpClient.delete<IResourceFormat>(
          `${this.resourceServiceUrl}${resourceId}/formats/${rfId}`,
          { withCredentials: true },
        );
      }),
    );
  }

  /* PAGINATION */
  paginationChanged(limit: number, pageNumber: number) {
    this.limit = limit;
    this.pageNumber = pageNumber;
    this._searchChangeSubject.next();
  }

  reverseSortOrder(): void {
    if (this.sortOptions.order === 'asc') {
      this.sortOptions.order = 'desc';
    } else {
      this.sortOptions.order = 'asc';
    }
  }

  get searchChange$(): Observable<string> {
    return this._searchChangeSubject.asObservable();
  }
}
